@@ -1,4 +1,4 @@
-from spin_phonon_suite.cli import *
+from molvis.cli import *
 """
 molvis is a wrapper package for making
 html chemical visualisers using 3dmol.js
