@@ -1,3 +1,4 @@
+from re import L
 import numpy as np
 import xyz_py as xyzp
 
@@ -5,21 +6,64 @@ from . import utils as ut
 
 
 class Viewer():
+    """
+    Wrapper for 3Dmol.js GLViewer class
+
+    Parameters
+    ----------
+    objects : list[str], optional
+        List of string objects/commands added to viewer callback function.
+        e.g. output from `core.Model`
+        ```
+        ms1 = core.Model.from_xyz(xyz_file)
+        viewer_1 = core.Viewer(objects=[ms1])
+        ```
+
+    extra_div_args : dict, optional
+        Extra arguments passed to viewer div
+    extra_style_args : dict, optional
+        Extra arguments passed to style property of viewer div
+    rc_download : bool, default True
+        Enables right-click download of model on webpage
+    
+    Attributes
+    ----------
+    objects : list[str], optional
+        List of molvis objects added to viewer callback function whose
+        __repr__ will be called via str()
+        e.g. `core.Model`
+        ```
+        ms1 = core.Model.from_xyz(xyz_file)
+        viewer_1 = core.Viewer(objects=[ms1])
+        ```
+    div_id : str
+        viewer div id string
+    height : str
+        viewer div height as string
+    width : str
+        viewer div width as string
+    position : str
+        viewer div position as string
+    style : dict
+        viewer div style, contains height, width and position. Can
+        be overwritten completely.
+    div_args : dict
+        viewer div properties, including div_id, style, class, and
+        background colour
+
+    """
 
     def __init__(self, extra_div_args={}, extra_style_args={},
                  rc_download=True, objects=[]):
+        
+        self.div_args = {}
+        self.style = {}
 
         self.div_id = "viewer"
         self.height = "100vh"
         self.width = "100vw"
         self.position = "relative"
 
-        self.style = {
-            "height": self.height,
-            "width": self.width,
-            "position": self.position
-        }
-
         if len(extra_style_args):
             for esa_key, esa_val in extra_style_args.items():
                 self.style[esa_key] = esa_val
@@ -27,7 +71,6 @@ class Viewer():
         self.div_args = {
             "div_id": self.div_id,
             "style": self.style,
-            "class": "viewer_3Dmoljs",
             "data-backgroundcolor": "0xf6f6f6"
         }
 
@@ -45,6 +88,105 @@ class Viewer():
 
         return
 
+    @property
+    def div_id(self):
+        return self._div_id
+
+    @div_id.setter
+    def div_id(self, div_id: str):
+        if not isinstance(div_id, str):
+            raise TypeError("div_id should be str")
+        self._div_id = div_id
+        self.div_args["div_id"] = self._div_id
+        return
+
+    @property
+    def height(self):
+        return self._height
+
+    @height.setter
+    def height(self, height: str):
+        if not isinstance(height, str):
+            raise TypeError("height should be str")
+        self._height = height
+        self._style["height"] = self._height
+        return
+    
+    @property
+    def width(self):
+        return self._width
+
+    @width.setter
+    def width(self, width: str):
+        if not isinstance(width, str):
+            raise TypeError("width should be str")
+        self._width = width
+        self._style["width"] = self._width
+        return
+
+    @property
+    def position(self):
+        return self._position
+
+    @position.setter
+    def position(self, position: str):
+        if not isinstance(position, str):
+            raise TypeError("position should be str")
+        self._position = position
+        self._style["position"] = self._position
+        return
+
+    @property
+    def style(self):
+        return self._style
+
+    @style.setter
+    def style(self, style: dict):
+        if not isinstance(style, dict):
+            raise TypeError("style should be dict")
+        self._style = style
+
+        # Set private height, width, position if present
+        if "height" in self._style.keys():
+            self._height = self._style["height"]
+        if "width" in self._style.keys():
+            self._width = self._style["width"]
+        if "position" in self._style.keys():
+            self._position = self._style["position"]
+        return
+
+    @property
+    def div_args(self):
+        return self._div_args
+
+    @div_args.setter
+    def div_args(self, div_args: dict):
+        if not isinstance(div_args, dict):
+            raise TypeError("div_args should be dict")
+        self._div_args = div_args
+        # Class cannot change
+        self._div_args["class"] = "viewer_3Dmoljs"
+
+        # Set private div_id, background colour, style if present
+        if "style" in self._div_args.keys():
+            self.style = self._div_args["style"]
+        if "div_id" in self._div_args.keys():
+            self._div_id = self._div_args["div_id"]
+        return
+
+
+    @property
+    def rc_download(self):
+        return self._rc_download
+
+    @rc_download.setter
+    def rc_download(self, rc_download: bool):
+        if not isinstance(rc_download, bool):
+            raise TypeError("rc_download should be bool")
+        self._rc_download = rc_download
+        return
+
+
     def add_object(self, object):
         self.objects.append(object)
         return
@@ -109,17 +251,6 @@ class Viewer():
         return string
 
 
-class Atom():
-
-    def __init__(self, style, colour, opacity, serial) -> None:
-
-        self.style = style
-        self.colour = colour
-        self.opacity = opacity
-        self.serial = serial
-
-        return
-
 
 class Model():
     def __init__(self, labels, coords, **kwargs) -> None:
@@ -141,10 +272,9 @@ class Model():
 
         self.model_js_var = "m"
         # Atom representation type, "stick", "sphere", "cartoon"
-        self._atom_rep = [
-            "stick"
-            for _ in range(self.n_atoms)
-        ]
+        self.atom_rep = "stick"
+
+        # Dictionary, one per atom specifying a single style for that atom
         self.atom_styles = [
             {}
             for _ in range(self.n_atoms)
@@ -157,6 +287,9 @@ class Model():
             for label in self.labels_nn
         ]
         self.atom_opacities = 1.
+        self.atom_radii = 1.
+
+        self.extra_styles = []
 
         # Set kwargs
         for key in kwargs.keys():
@@ -201,7 +334,7 @@ class Model():
             raise TypeError("atom_opacities should be either list or float")
         elif isinstance(atom_opacities, float):
             self._atom_opacities = [
-                atom_opacities for it in range(self.n_atoms)
+                atom_opacities for _ in range(self.n_atoms)
             ]
         elif len(atom_opacities) == self.n_atoms:
             self._atom_opacities = atom_opacities
@@ -210,6 +343,28 @@ class Model():
 
         for it in range(self.n_atoms):
             self.atom_styles[it]["opacity"] = self._atom_opacities[it]
+        return
+
+    @property
+    def atom_radii(self):
+        return self._atom_radii
+
+    @atom_radii.setter
+    def atom_radii(self, atom_radii: list):
+
+        if not isinstance(atom_radii, (list, float)):
+            raise TypeError("atom_radii should be either list or float")
+        elif isinstance(atom_radii, float):
+            self._atom_radii = [
+                atom_radii for _ in range(self.n_atoms)
+            ]
+        elif len(atom_radii) == self.n_atoms:
+            self._atom_radii = atom_radii
+        else:
+            raise ValueError("Number of radii should be 1 or n_atoms")
+
+        for it in range(self.n_atoms):
+            self.atom_styles[it]["radius"] = self._atom_radii[it]
 
         return
 
@@ -249,6 +404,9 @@ class Model():
 
         set_styles = self.gen_set_style()
 
+        if len(self.extra_styles):
+            set_styles = self.merge_styles(set_styles, self.extra_styles)
+
         for set_style in set_styles:
             model_str += "            {}.setStyle(".format(
                 self.model_js_var
@@ -320,6 +478,18 @@ class Model():
 
         return set_style
 
+    @staticmethod
+    def merge_styles(style_1, style_2):
+
+        new_style = [
+            [
+                s1[0],
+                dict(s1[1], **s2[1])
+            ]
+            for s1, s2 in zip(style_1, style_2)
+        ]
+
+        return new_style
 
 class HtmlPage():
     """
